@@ -29,23 +29,6 @@ G_DEFINE_TYPE(FuVliUsbhubRtd21xxDevice, fu_vli_usbhub_rtd21xx_device, FU_TYPE_DE
 #define ISP_DATA_BLOCKSIZE 30
 #define ISP_PACKET_SIZE	   32
 
-typedef enum {
-	ISP_STATUS_BUSY = 0xBB,		/* host must wait for device */
-	ISP_STATUS_IDLE_SUCCESS = 0x11, /* previous command was OK */
-	ISP_STATUS_IDLE_FAILURE = 0x12, /* previous command failed */
-} IspStatus;
-
-typedef enum {
-	ISP_CMD_ENTER_FW_UPDATE = 0x01,
-	ISP_CMD_GET_PROJECT_ID_ADDR = 0x02,
-	ISP_CMD_SYNC_IDENTIFY_CODE = 0x03,
-	ISP_CMD_GET_FW_INFO = 0x04,
-	ISP_CMD_FW_UPDATE_START = 0x05,
-	ISP_CMD_FW_UPDATE_ISP_DONE = 0x06,
-	ISP_CMD_FW_UPDATE_EXIT = 0x07,
-	ISP_CMD_FW_UPDATE_RESET = 0x08,
-} IspCmd;
-
 static gboolean
 fu_vli_usbhub_rtd21xx_device_i2c_write(FuVliUsbhubDevice *self,
 				       guint8 target_addr,
@@ -122,8 +105,12 @@ fu_vli_usbhub_rtd21xx_device_read_status_raw(FuVliUsbhubRtd21xxDevice *self,
 					     guint8 *status,
 					     GError **error)
 {
-	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(FU_DEVICE(self)));
+	FuVliUsbhubDevice *parent;
 	guint8 buf[] = {0x00};
+
+	parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(FU_DEVICE(self), error));
+	if (parent == NULL)
+		return FALSE;
 	if (!fu_vli_usbhub_rtd21xx_device_i2c_read(parent,
 						   UC_FOREGROUND_TARGET_ADDR,
 						   UC_FOREGROUND_STATUS,
@@ -143,7 +130,7 @@ fu_vli_usbhub_rtd21xx_device_read_status_cb(FuDevice *device, gpointer user_data
 	guint8 status = 0xfd;
 	if (!fu_vli_usbhub_rtd21xx_device_read_status_raw(self, &status, error))
 		return FALSE;
-	if (status == ISP_STATUS_BUSY) {
+	if (status == FU_VLI_USBHUB_RTD21XX_ISP_STATUS_BUSY) {
 		g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_INTERNAL, "status was 0x%02x", status);
 		return FALSE;
 	}
@@ -165,11 +152,14 @@ fu_vli_usbhub_rtd21xx_device_read_status(FuVliUsbhubRtd21xxDevice *self,
 static gboolean
 fu_vli_usbhub_rtd21xx_device_ensure_version_unlocked(FuVliUsbhubRtd21xxDevice *self, GError **error)
 {
-	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(FU_DEVICE(self)));
+	FuVliUsbhubDevice *parent;
 	guint8 buf_rep[7] = {0x00};
-	guint8 buf_req[] = {ISP_CMD_GET_FW_INFO};
+	guint8 buf_req[] = {FU_VLI_USBHUB_RTD21XX_ISP_CMD_GET_FW_INFO};
 	g_autofree gchar *version = NULL;
 
+	parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(FU_DEVICE(self), error));
+	if (parent == NULL)
+		return FALSE;
 	if (!fu_vli_usbhub_rtd21xx_device_i2c_write(parent,
 						    UC_FOREGROUND_TARGET_ADDR,
 						    UC_FOREGROUND_OPCODE,
@@ -221,8 +211,12 @@ fu_vli_usbhub_rtd21xx_device_setup(FuDevice *device, GError **error)
 static gboolean
 fu_vli_usbhub_rtd21xx_device_detach_raw(FuVliUsbhubRtd21xxDevice *self, GError **error)
 {
-	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(FU_DEVICE(self)));
+	FuVliUsbhubDevice *parent;
 	guint8 buf[] = {0x03};
+
+	parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(FU_DEVICE(self), error));
+	if (parent == NULL)
+		return FALSE;
 	if (!fu_vli_usbhub_rtd21xx_device_i2c_write(parent, 0x6A, 0x31, buf, sizeof(buf), error)) {
 		g_prefix_error_literal(error, "failed to detach: ");
 		return FALSE;
@@ -239,7 +233,7 @@ fu_vli_usbhub_rtd21xx_device_detach_cb(FuDevice *device, gpointer user_data, GEr
 		return FALSE;
 	if (!fu_vli_usbhub_rtd21xx_device_read_status_raw(self, &status, error))
 		return FALSE;
-	if (status != ISP_STATUS_IDLE_SUCCESS) {
+	if (status != FU_VLI_USBHUB_RTD21XX_ISP_STATUS_IDLE_SUCCESS) {
 		g_set_error(error,
 			    FWUPD_ERROR,
 			    FWUPD_ERROR_INTERNAL,
@@ -253,10 +247,13 @@ fu_vli_usbhub_rtd21xx_device_detach_cb(FuDevice *device, gpointer user_data, GEr
 static gboolean
 fu_vli_usbhub_rtd21xx_device_detach(FuDevice *device, FuProgress *progress, GError **error)
 {
-	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(device));
+	FuDevice *parent;
 	g_autoptr(FuDeviceLocker) locker = NULL;
 
 	/* open device */
+	parent = fu_device_get_parent(device, error);
+	if (parent == NULL)
+		return FALSE;
 	locker = fu_device_locker_new(parent, error);
 	if (locker == NULL)
 		return FALSE;
@@ -271,12 +268,15 @@ fu_vli_usbhub_rtd21xx_device_detach(FuDevice *device, FuProgress *progress, GErr
 static gboolean
 fu_vli_usbhub_rtd21xx_device_attach(FuDevice *device, FuProgress *progress, GError **error)
 {
-	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(device));
-	guint8 buf[] = {ISP_CMD_FW_UPDATE_RESET};
+	FuVliUsbhubDevice *parent;
+	guint8 buf[] = {FU_VLI_USBHUB_RTD21XX_ISP_CMD_FW_UPDATE_RESET};
 	g_autoptr(FuDeviceLocker) locker = NULL;
 
 	/* open device */
-	locker = fu_device_locker_new(parent, error);
+	parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(device, error));
+	if (parent == NULL)
+		return FALSE;
+	locker = fu_device_locker_new(FU_DEVICE(parent), error);
 	if (locker == NULL)
 		return FALSE;
 	if (!fu_vli_usbhub_rtd21xx_device_i2c_write(parent,
@@ -301,7 +301,7 @@ fu_vli_usbhub_rtd21xx_device_write_firmware(FuDevice *device,
 					    FwupdInstallFlags flags,
 					    GError **error)
 {
-	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(device));
+	FuVliUsbhubDevice *parent;
 	FuVliUsbhubRtd21xxDevice *self = FU_VLI_USBHUB_RTD21XX_DEVICE(device);
 	guint32 project_addr;
 	guint8 project_id_count;
@@ -319,7 +319,10 @@ fu_vli_usbhub_rtd21xx_device_write_firmware(FuDevice *device,
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 10, NULL);
 
 	/* open device */
-	locker = fu_device_locker_new(parent, error);
+	parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(device, error));
+	if (parent == NULL)
+		return FALSE;
+	locker = fu_device_locker_new(FU_DEVICE(parent), error);
 	if (locker == NULL)
 		return FALSE;
 
@@ -329,7 +332,7 @@ fu_vli_usbhub_rtd21xx_device_write_firmware(FuDevice *device,
 		return FALSE;
 
 	/* enable ISP high priority */
-	write_buf[0] = ISP_CMD_ENTER_FW_UPDATE;
+	write_buf[0] = FU_VLI_USBHUB_RTD21XX_ISP_CMD_ENTER_FW_UPDATE;
 	write_buf[1] = 0x01;
 	if (!fu_vli_usbhub_rtd21xx_device_i2c_write(parent,
 						    UC_FOREGROUND_TARGET_ADDR,
@@ -344,7 +347,7 @@ fu_vli_usbhub_rtd21xx_device_write_firmware(FuDevice *device,
 		return FALSE;
 
 	/* get project ID address */
-	write_buf[0] = ISP_CMD_GET_PROJECT_ID_ADDR;
+	write_buf[0] = FU_VLI_USBHUB_RTD21XX_ISP_CMD_GET_PROJECT_ID_ADDR;
 	if (!fu_vli_usbhub_rtd21xx_device_i2c_write(parent,
 						    UC_FOREGROUND_TARGET_ADDR,
 						    UC_FOREGROUND_OPCODE,
@@ -366,8 +369,12 @@ fu_vli_usbhub_rtd21xx_device_write_firmware(FuDevice *device,
 		g_prefix_error_literal(error, "failed to read project ID: ");
 		return FALSE;
 	}
-	if (read_buf[0] != ISP_STATUS_IDLE_SUCCESS) {
-		g_prefix_error(error, "failed project ID with error 0x%02x: ", read_buf[0]);
+	if (read_buf[0] != FU_VLI_USBHUB_RTD21XX_ISP_STATUS_IDLE_SUCCESS) {
+		g_set_error(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_INTERNAL,
+			    "failed project ID with error 0x%02x",
+			    read_buf[0]);
 		return FALSE;
 	}
 
@@ -380,7 +387,7 @@ fu_vli_usbhub_rtd21xx_device_write_firmware(FuDevice *device,
 				    error))
 		return FALSE;
 	project_id_count = read_buf[5];
-	write_buf[0] = ISP_CMD_SYNC_IDENTIFY_CODE;
+	write_buf[0] = FU_VLI_USBHUB_RTD21XX_ISP_CMD_SYNC_IDENTIFY_CODE;
 	if (!fu_input_stream_read_safe(stream,
 				       write_buf,
 				       sizeof(write_buf),
@@ -404,7 +411,7 @@ fu_vli_usbhub_rtd21xx_device_write_firmware(FuDevice *device,
 		return FALSE;
 
 	/* background FW update start command */
-	write_buf[0] = ISP_CMD_FW_UPDATE_START;
+	write_buf[0] = FU_VLI_USBHUB_RTD21XX_ISP_CMD_FW_UPDATE_START;
 	fu_memwrite_uint16(write_buf + 1, ISP_DATA_BLOCKSIZE, G_BIG_ENDIAN);
 	if (!fu_vli_usbhub_rtd21xx_device_i2c_write(parent,
 						    UC_FOREGROUND_TARGET_ADDR,
@@ -456,7 +463,7 @@ fu_vli_usbhub_rtd21xx_device_write_firmware(FuDevice *device,
 	/* update finish command */
 	if (!fu_vli_usbhub_rtd21xx_device_read_status(self, NULL, error))
 		return FALSE;
-	write_buf[0] = ISP_CMD_FW_UPDATE_ISP_DONE;
+	write_buf[0] = FU_VLI_USBHUB_RTD21XX_ISP_CMD_FW_UPDATE_ISP_DONE;
 	if (!fu_vli_usbhub_rtd21xx_device_i2c_write(parent,
 						    UC_FOREGROUND_TARGET_ADDR,
 						    UC_FOREGROUND_OPCODE,
@@ -471,7 +478,7 @@ fu_vli_usbhub_rtd21xx_device_write_firmware(FuDevice *device,
 	/* exit background-fw mode */
 	if (!fu_vli_usbhub_rtd21xx_device_read_status(self, NULL, error))
 		return FALSE;
-	write_buf[0] = ISP_CMD_FW_UPDATE_EXIT;
+	write_buf[0] = FU_VLI_USBHUB_RTD21XX_ISP_CMD_FW_UPDATE_EXIT;
 	if (!fu_vli_usbhub_rtd21xx_device_i2c_write(parent,
 						    UC_FOREGROUND_TARGET_ADDR,
 						    UC_FOREGROUND_OPCODE,
@@ -494,10 +501,13 @@ fu_vli_usbhub_rtd21xx_device_write_firmware(FuDevice *device,
 static gboolean
 fu_vli_usbhub_rtd21xx_device_reload(FuDevice *device, GError **error)
 {
-	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(device));
+	FuDevice *parent;
 	g_autoptr(FuDeviceLocker) locker = NULL;
 
 	/* open parent device */
+	parent = fu_device_get_parent(device, error);
+	if (parent == NULL)
+		return FALSE;
 	locker = fu_device_locker_new(parent, error);
 	if (locker == NULL)
 		return FALSE;
@@ -508,7 +518,7 @@ static gboolean
 fu_vli_usbhub_rtd21xx_device_probe(FuDevice *device, GError **error)
 {
 	FuVliDeviceKind device_kind = FU_VLI_DEVICE_KIND_RTD21XX;
-	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(device));
+	FuVliUsbhubDevice *parent = FU_VLI_USBHUB_DEVICE(fu_device_get_parent(device, NULL));
 
 	fu_device_set_name(device, fu_vli_device_kind_to_string(device_kind));
 	if (parent != NULL) {
@@ -523,7 +533,7 @@ fu_vli_usbhub_rtd21xx_device_probe(FuDevice *device, GError **error)
 }
 
 static void
-fu_vli_usbhub_rtd21xx_device_set_progress(FuDevice *self, FuProgress *progress)
+fu_vli_usbhub_rtd21xx_device_set_progress(FuDevice *device, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_add_flag(progress, FU_PROGRESS_FLAG_GUESSED);
